# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" An action manager item that represents an actual action. """


from traits.api import Any, Instance, List, Property, Str, observe


from pyface.action.action import Action
from pyface.action.action_manager_item import ActionManagerItem

# Import the toolkit specific versions of the internal classes.
from pyface.toolkit import toolkit_object

_MenuItem = toolkit_object("action.action_item:_MenuItem")
_Tool = toolkit_object("action.action_item:_Tool")
_PaletteTool = toolkit_object("action.action_item:_PaletteTool")


class ActionItem(ActionManagerItem):
    """ An action manager item that represents an actual action. """

    # 'ActionManagerItem' interface ----------------------------------------

    #: The item's unique identifier ('unique' in this case means unique within
    #: its group).
    id = Property(Str)

    # 'ActionItem' interface -----------------------------------------------

    #: The action!
    action = Instance(Action)

    #: The toolkit specific control created for this item.
    control = Any()

    #: The toolkit specific Id of the control created for this item.
    #
    #: We have to keep the Id as well as the control because wx tool bar tools
    #: are created as 'wxObjectPtr's which do not have Ids, and the Id is
    #: required to manipulate the state of a tool via the tool bar 8^(
    # FIXME v3: Why is this part of the public interface?
    control_id = Any()

    # Private interface ----------------------------------------------------

    #: All of the internal instances that wrap this item.
    _wrappers = List(Any)

    # ------------------------------------------------------------------------
    # 'ActionManagerItem' interface.
    # ------------------------------------------------------------------------

    # Trait properties -----------------------------------------------------

    def _get_id(self):
        return self.action.id

    # Trait change handlers ------------------------------------------------

    def _enabled_changed(self, trait_name, old, new):
        self.action.enabled = new

    def _visible_changed(self, trait_name, old, new):
        self.action.visible = new

    @observe("_wrappers:items:control")
    def _on_destroy(self, event):
        """ Handle the destruction of the wrapper. """
        if event.new is None:
            self._wrappers.remove(event.object)

    # ------------------------------------------------------------------------
    # 'ActionItem' interface.
    # ------------------------------------------------------------------------

    def add_to_menu(self, parent, menu, controller):
        """ Add the item to a menu.

        Parameters
        ----------
        parent : toolkit control
            The parent of the new menu item control.
        menu : toolkit menu
            The menu to add the action item to.
        controller : ActionController instance or None
            The controller to use.
        """
        if (controller is None) or controller.can_add_to_menu(self.action):
            wrapper = _MenuItem(parent, menu, self, controller)

            # fixme: Martin, who uses this information?
            if controller is None:
                self.control = wrapper.control
                self.control_id = wrapper.control_id

            self._wrappers.append(wrapper)

    def add_to_toolbar(
        self, parent, tool_bar, image_cache, controller, show_labels=True
    ):
        """ Adds the item to a tool bar.

        Parameters
        ----------
        parent : toolkit control
            The parent of the new menu item control.
        tool_bar : toolkit toolbar
            The toolbar to add the action item to.
        image_cache : ImageCache instance
            The image cache for resized images.
        controller : ActionController instance or None
            The controller to use.
        show_labels : bool
            Should the toolbar item show a label.
        """
        if (controller is None) or controller.can_add_to_toolbar(self.action):
            wrapper = _Tool(
                parent, tool_bar, image_cache, self, controller, show_labels
            )

            # fixme: Martin, who uses this information?
            if controller is None:
                self.control = wrapper.control
                self.control_id = wrapper.control_id

            self._wrappers.append(wrapper)

    def add_to_palette(self, tool_palette, image_cache, show_labels=True):
        """ Adds the item to a tool palette.

        Parameters
        ----------
        parent : toolkit control
            The parent of the new menu item control.
        tool_palette : toolkit tool palette
            The tool palette to add the action item to.
        image_cache : ImageCache instance
            The image cache for resized images.
        show_labels : bool
            Should the toolbar item show a label.
        """
        wrapper = _PaletteTool(tool_palette, image_cache, self, show_labels)
        self._wrappers.append(wrapper)

    def destroy(self):
        """ Called when the action is no longer required.

        By default this method calls 'destroy' on the action itself.
        """
        self.action.destroy()



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


""" Enthought pyface package component
"""


import sys
import wx


from traits.api import Bool, Enum, Int, provides, Str


from pyface.i_dialog import IDialog, MDialog
from pyface.constant import OK, CANCEL, YES, NO
from .window import Window


# Map wx dialog related constants to the pyface equivalents.
_RESULT_MAP = {
    wx.ID_OK: OK,
    wx.ID_CANCEL: CANCEL,
    wx.ID_YES: YES,
    wx.ID_NO: NO,
    wx.ID_CLOSE: CANCEL,
    # There seems to be a bug in wx.SingleChoiceDialog that allows it to return
    # 0 when it is closed via the window (closing it via the buttons works just
    # fine).
    0: CANCEL,
}


@provides(IDialog)
class Dialog(MDialog, Window):
    """ The toolkit specific implementation of a Dialog.  See the IDialog
    interface for the API documentation.
    """

    # 'IDialog' interface -------------------------------------------------#

    cancel_label = Str()

    help_id = Str()

    help_label = Str()

    ok_label = Str()

    resizeable = Bool(True)

    return_code = Int(OK)

    style = Enum("modal", "nonmodal")

    # 'IWindow' interface -------------------------------------------------#

    title = Str("Dialog")

    # ------------------------------------------------------------------------
    # Protected 'IDialog' interface.
    # ------------------------------------------------------------------------

    def _create_buttons(self, parent):
        sizer = wx.StdDialogButtonSizer()

        # The 'OK' button.
        if self.ok_label:
            label = self.ok_label
        else:
            label = "OK"

        self._wx_ok = ok = wx.Button(parent, wx.ID_OK, label)
        ok.SetDefault()
        parent.Bind(wx.EVT_BUTTON, self._wx_on_ok, id=wx.ID_OK)
        sizer.AddButton(ok)

        # The 'Cancel' button.
        if self.cancel_label:
            label = self.cancel_label
        else:
            label = "Cancel"

        self._wx_cancel = cancel = wx.Button(parent, wx.ID_CANCEL, label)
        parent.Bind(wx.EVT_BUTTON, self._wx_on_cancel, id=wx.ID_CANCEL)
        sizer.AddButton(cancel)

        # The 'Help' button.
        if len(self.help_id) > 0:
            if self.help_label:
                label = self.help_label
            else:
                label = "Help"

            help = wx.Button(parent, wx.ID_HELP, label)
            parent.Bind(wx.EVT_BUTTON, self._wx_on_help, id=wx.ID_HELP)
            sizer.AddButton(help)

        sizer.Realize()
        return sizer

    def _create_contents(self, parent):
        sizer = wx.BoxSizer(wx.VERTICAL)
        parent.SetSizer(sizer)
        parent.SetAutoLayout(True)

        # The 'guts' of the dialog.
        dialog_area = self._create_dialog_area(parent)
        sizer.Add(dialog_area, 1, wx.EXPAND | wx.ALL, 5)

        # The buttons.
        buttons = self._create_buttons(parent)
        sizer.Add(buttons, 0, wx.ALIGN_RIGHT | wx.ALL, 5)

        # Resize the dialog to match the sizer's minimal size.
        if self.size != (-1, -1):
            parent.SetSize(self.size)
        else:
            sizer.Fit(parent)

        parent.CentreOnParent()

    def _create_dialog_area(self, parent):
        panel = wx.Panel(parent, -1)
        panel.SetBackgroundColour("red")
        panel.SetSize((100, 200))

        return panel

    def _show_modal(self):
        if sys.platform == "darwin":
            # Calling Show(False) is needed on the Mac for the modal dialog
            # to show up at all.
            self.control.Show(False)
        return _RESULT_MAP[self.control.ShowModal()]

    # ------------------------------------------------------------------------
    # Protected 'IWidget' interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        style = wx.DEFAULT_DIALOG_STYLE | wx.CLIP_CHILDREN

        if self.resizeable:
            style |= wx.RESIZE_BORDER

        return wx.Dialog(
            parent, -1, self.title, self.position, self.size, style
        )

    # wx event handlers ----------------------------------------------------

    def _wx_on_ok(self, event):
        """ Called when the 'OK' button is pressed. """

        self.return_code = OK

        # Let the default handler close the dialog appropriately.
        event.Skip()

    def _wx_on_cancel(self, event):
        """ Called when the 'Cancel' button is pressed. """

        self.return_code = CANCEL

        # Let the default handler close the dialog appropriately.
        event.Skip()

    def _wx_on_help(self, event):
        """ Called when the 'Help' button is pressed. """
        pass



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The spin field interface. """


from traits.api import HasTraits, Int, Property, Range, Tuple

from pyface.fields.i_field import IField


class ISpinField(IField):
    """ The spin field interface.

    This is for spinners holding integer values.
    """

    #: The current value of the spinner
    value = Range(low="minimum", high="maximum")

    #: The bounds of the spinner
    bounds = Tuple(Int, Int)

    #: The minimum value
    minimum = Property(Int, observe="bounds")

    #: The maximum value
    maximum = Property(Int, observe="bounds")


class MSpinField(HasTraits):

    #: The current value of the spinner
    value = Range(low="minimum", high="maximum")

    #: The bounds for the spinner
    bounds = Tuple(Int, Int)

    #: The minimum value for the spinner
    minimum = Property(Int, observe="bounds")

    #: The maximum value for the spinner
    maximum = Property(Int, observe="bounds")

    # ------------------------------------------------------------------------
    # object interface
    # ------------------------------------------------------------------------

    def __init__(self, **traits):
        value = traits.pop("value", None)
        if "bounds" in traits:
            traits["value"] = traits["bounds"][0]
        super(MSpinField, self).__init__(**traits)
        if value is not None:
            self.value = value

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    def _initialize_control(self):
        super(MSpinField, self)._initialize_control()
        self._set_control_bounds(self.bounds)
        self._set_control_value(self.value)

    def _add_event_listeners(self):
        """ Set up toolkit-specific bindings for events """
        super(MSpinField, self)._add_event_listeners()
        self.observe(self._bounds_updated, "bounds", dispatch="ui")
        if self.control is not None:
            self._observe_control_value()

    def _remove_event_listeners(self):
        """ Remove toolkit-specific bindings for events """
        if self.control is not None:
            self._observe_control_value(remove=True)
        self.observe(
            self._bounds_updated, "bounds", dispatch="ui", remove=True
        )
        super(MSpinField, self)._remove_event_listeners()

    # Toolkit control interface ---------------------------------------------

    def _get_control_bounds(self):
        """ Toolkit specific method to get the control's bounds. """
        raise NotImplementedError()

    def _set_control_bounds(self, bounds):
        """ Toolkit specific method to set the control's bounds. """
        raise NotImplementedError()

    # Trait property handlers -----------------------------------------------

    def _get_minimum(self):
        return self.bounds[0]

    def _set_minimum(self, value):
        if value > self.maximum:
            self.bounds = (value, value)
        else:
            self.bounds = (value, self.maximum)
        if value > self.value:
            self.value = value

    def _get_maximum(self):
        return self.bounds[1]

    def _set_maximum(self, value):
        if value < self.minimum:
            self.bounds = (value, value)
        else:
            self.bounds = (self.minimum, value)
        if value < self.value:
            self.value = value

    # Trait defaults --------------------------------------------------------

    def _value_default(self):
        return self.bounds[0]

    # Trait change handlers --------------------------------------------------

    def _bounds_updated(self, event):
        if self.control is not None:
            self._set_control_bounds(self.bounds)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from traits.testing.api import UnittestTools

from ..action import Action
from ..action_item import ActionItem
from ..action_manager import ActionManager
from ..group import Group


class TestActionItem(unittest.TestCase, UnittestTools):
    def setUp(self):
        # test whether function is called by updating list
        # XXX should really use mock
        self.memo = []

        def perform():
            self.memo.append("called")

        self.perform = perform

        self.action = Action(name="Test", on_perform=perform)
        self.action_item = ActionItem(action=self.action)
        self.group = Group(id="test")

    def test_init_group(self):
        action_manager = ActionManager(self.group)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [self.group, default_group])
        self.assertEqual(self.group.parent, action_manager)

    def test_init_string(self):
        action_manager = ActionManager("Test")
        default_group = action_manager._get_default_group()
        self.assertEqual(len(action_manager.groups), 2)
        self.assertEqual(action_manager.groups[0].id, "Test")
        self.assertEqual(action_manager.groups[-1], default_group)
        self.assertEqual(action_manager.groups[0].parent, action_manager)

    def test_init_action_item(self):
        action_manager = ActionManager(self.action_item)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [default_group])
        self.assertEqual(default_group.items, [self.action_item])
        self.assertEqual(self.action_item.parent, default_group)

    def test_init_group_action_item(self):
        action_manager = ActionManager(self.group, self.action_item)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [self.group, default_group])
        self.assertEqual(self.group.items, [self.action_item])
        self.assertEqual(self.action_item.parent, self.group)

    def test_init_action(self):
        action_manager = ActionManager(self.action)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [default_group])
        self.assertEqual(default_group.items[0].action, self.action)
        self.assertEqual(default_group.items[0].parent, default_group)

    def test_init_nothing(self):
        action_manager = ActionManager()
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [default_group])
        self.assertEqual(len(default_group.items), 0)

    def test_append(self):
        action_manager = ActionManager()
        action_manager.append(self.group)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [default_group, self.group])
        self.assertEqual(self.group.parent, action_manager)

    def test_append_2(self):
        action_manager = ActionManager(self.group)
        group2 = Group()
        action_manager.append(group2)
        default_group = action_manager._get_default_group()
        self.assertEqual(
            action_manager.groups, [self.group, default_group, group2]
        )
        self.assertEqual(group2.parent, action_manager)

    def test_append_string(self):
        action_manager = ActionManager(self.group)
        action_manager.append("Test string")
        self.assertEqual(len(action_manager.groups), 3)
        self.assertEqual(action_manager.groups[2].id, "Test string")

    def test_append_item(self):
        action_manager = ActionManager()
        action_manager.append(self.action_item)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [default_group])
        self.assertEqual(default_group.items, [self.action_item])

    def test_append_item_2(self):
        action_manager = ActionManager()
        action_manager.append(self.group)
        action_manager.append(self.action_item)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [default_group, self.group])
        self.assertEqual(default_group.items, [self.action_item])

    def test_append_item_order(self):
        # Regression test for enthought/pyface#289
        expected = [
            self.action_item,
            ActionItem(action=Action(name="Test2")),
            ActionItem(action=Action(name="Test3")),
        ]
        action_manager = ActionManager()
        for item in expected:
            action_manager.append(item)

        default_group = action_manager._get_default_group()
        self.assertEqual(default_group.items, expected)

    def test_destroy(self):
        action_manager = ActionManager(self.group)
        # XXX items doesn't fire a change event.  Should it?
        # XXX should mock group to ensure that destroy is called
        action_manager.destroy()

    def test_insert(self):
        action_manager = ActionManager()
        action_manager.insert(0, self.group)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [self.group, default_group])
        self.assertEqual(self.group.parent, action_manager)

    def test_insert_2(self):
        action_manager = ActionManager(self.group)
        group2 = Group()
        action_manager.insert(1, group2)
        default_group = action_manager._get_default_group()
        self.assertEqual(
            action_manager.groups, [self.group, group2, default_group]
        )
        self.assertEqual(group2.parent, action_manager)

    def test_insert_string(self):
        action_manager = ActionManager(self.group)
        action_manager.insert(0, "Test string")
        self.assertEqual(len(action_manager.groups), 3)
        self.assertEqual(action_manager.groups[0].id, "Test string")

    def test_insert_item(self):
        action_manager = ActionManager()
        action_manager.insert(0, self.action_item)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [default_group])
        self.assertEqual(default_group.items, [self.action_item])

    def test_insert_item_2(self):
        action_manager = ActionManager()
        action_manager.append(self.group)
        action_manager.insert(0, self.action_item)
        default_group = action_manager._get_default_group()
        self.assertEqual(action_manager.groups, [default_group, self.group])
        self.assertEqual(default_group.items, [self.action_item])

    def test_find_group(self):
        action_manager = ActionManager(self.group)
        group = action_manager.find_group("test")
        self.assertEqual(group, self.group)

    def test_find_group_missing(self):
        action_manager = ActionManager(self.group)
        group = action_manager.find_group("not here")
        self.assertIsNone(group)

    def test_find_item(self):
        self.group.append(self.action_item)
        action_manager = ActionManager(self.group)
        item = action_manager.find_item("Test")
        self.assertEqual(item, self.action_item)

    def test_find_item_missing(self):
        self.group.append(self.action_item)
        action_manager = ActionManager(self.group)
        item = action_manager.find_item("Not here")
        self.assertIsNone(item)

    def test_find_item_hierarchy(self):
        action_manager = ActionManager(self.group)
        action_manager_2 = ActionManager(self.action_item, id="test2")
        self.group.append(action_manager_2)
        item = action_manager.find_item("test2/Test")
        self.assertEqual(item, self.action_item)

    def test_walk_hierarchy(self):
        action_manager = ActionManager(self.group)
        action_manager_2 = ActionManager(self.action_item, id="test2")
        self.group.append(action_manager_2)
        result = []
        action_manager.walk(result.append)
        self.assertEqual(
            result,
            [
                action_manager,
                self.group,
                action_manager_2,
                action_manager_2._get_default_group(),
                self.action_item,
                action_manager._get_default_group(),
            ],
        )

    def test_enabled_changed(self):
        self.group.append(self.action_item)
        action_manager = ActionManager(self.group)
        action_manager.enabled = False
        self.assertFalse(self.group.enabled)
        self.assertFalse(self.action_item.enabled)
        self.assertFalse(self.action.enabled)
        action_manager.enabled = True
        self.assertTrue(self.group.enabled)
        self.assertTrue(self.action_item.enabled)
        self.assertTrue(self.action.enabled)

    def test_visible_changed(self):
        self.group.append(self.action_item)
        action_manager = ActionManager(self.group)
        action_manager.visible = False
        self.assertFalse(self.group.visible)
        # XXX group doesn't make items invisible
        # self.assertFalse(self.action_item.enabled)
        # self.assertFalse(self.action.enabled)
        action_manager.visible = True
        self.assertTrue(self.group.visible)
        # XXX group doesn't make items visible
        # self.assertTrue(self.action_item.enabled)
        # self.assertTrue(self.action.enabled)



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from unittest import TestCase

from pyface.data_view.index_manager import (
    IntIndexManager, Root, TupleIndexManager,
)


class IndexManagerMixin:

    def test_root_has_no_parent(self):
        with self.assertRaises(IndexError):
            self.index_manager.get_parent_and_row(Root)

    def test_root_to_sequence(self):
        result = self.index_manager.to_sequence(Root)

        self.assertEqual(result, ())

    def test_root_from_sequence(self):
        result = self.index_manager.from_sequence([])

        self.assertIs(result, Root)

    def test_root_id_round_trip(self):
        root_id = self.index_manager.id(Root)
        result = self.index_manager.from_id(root_id)

        self.assertIs(result, Root)

    def test_simple_sequence_round_trip(self):
        sequence = (5,)
        index = self.index_manager.from_sequence(sequence)
        result = self.index_manager.to_sequence(index)

        self.assertEqual(result, sequence)

    def test_simple_sequence_invalid(self):
        sequence = (-5,)
        with self.assertRaises(IndexError):
            self.index_manager.from_sequence(sequence)

    def test_simple_sequence_to_parent_row(self):
        sequence = (5,)
        index = self.index_manager.from_sequence(sequence)
        result = self.index_manager.get_parent_and_row(index)

        self.assertEqual(result, (Root, 5))

    def test_simple_row_round_trip(self):
        index = self.index_manager.create_index(Root, 5)
        result = self.index_manager.get_parent_and_row(index)

        self.assertEqual(result, (Root, 5))

    def test_simple_row_invalid(self):
        with self.assertRaises(IndexError):
            self.index_manager.create_index(Root, -5)

    def test_simple_row_to_sequence(self):
        index = self.index_manager.create_index(Root, 5)
        result = self.index_manager.to_sequence(index)

        self.assertEqual(result, (5,))

    def test_simple_id_round_trip(self):
        index = self.index_manager.create_index(Root, 5)
        id = self.index_manager.id(index)
        result = self.index_manager.from_id(id)

        self.assertEqual(result, index)


class TestIntIndexManager(IndexManagerMixin, TestCase):

    def setUp(self):
        super().setUp()
        self.index_manager = IntIndexManager()

    def tearDown(self):
        self.index_manager.reset()

    def test_create_index_root(self):
        result = self.index_manager.create_index(Root, 5)
        self.assertEqual(result, 5)

    def test_create_index_leaf(self):
        with self.assertRaises(RuntimeError):
            self.index_manager.create_index(5, 1)

    def test_create_index_negative(self):
        with self.assertRaises(IndexError):
            self.index_manager.create_index(Root, -5)


class TestTupleIndexManager(IndexManagerMixin, TestCase):

    def setUp(self):
        super().setUp()
        self.index_manager = TupleIndexManager()

    def tearDown(self):
        self.index_manager.reset()

    def test_complex_sequence_round_trip(self):
        sequence = (5, 6, 7, 8, 9, 10)
        index = self.index_manager.from_sequence(sequence)
        result = self.index_manager.to_sequence(index)

        self.assertEqual(result, sequence)

    def test_complex_sequence_identical_index(self):
        sequence = (5, 6, 7, 8, 9, 10)
        index_1 = self.index_manager.from_sequence(sequence[:])
        index_2 = self.index_manager.from_sequence(sequence[:])

        self.assertIs(index_1, index_2)

    def test_complex_sequence_to_parent_row(self):
        sequence = (5, 6, 7, 8, 9, 10)
        index = self.index_manager.from_sequence(sequence)

        parent, row = self.index_manager.get_parent_and_row(index)

        self.assertEqual(row, 10)
        self.assertIs(
            parent,
            self.index_manager.from_sequence((5, 6, 7, 8, 9))
        )

    def test_complex_index_round_trip(self):
        sequence = (5, 6, 7, 8, 9, 10)

        parent = Root
        for depth, row in enumerate(sequence):
            with self.subTest(depth=depth):
                index = self.index_manager.create_index(parent, row)
                result = self.index_manager.get_parent_and_row(index)
                self.assertIs(result[0], parent)
                self.assertEqual(result[1], row)
                parent = index

    def test_complex_index_create_index_identical(self):
        sequence = (5, 6, 7, 8, 9, 10)

        parent = Root
        for depth, row in enumerate(sequence):
            with self.subTest(depth=depth):
                index_1 = self.index_manager.create_index(parent, row)
                index_2 = self.index_manager.create_index(parent, row)
                self.assertIs(index_1, index_2)
                parent = index_1

    def test_complex_index_to_sequence(self):
        sequence = (5, 6, 7, 8, 9, 10)
        parent = Root
        for depth, row in enumerate(sequence):
            with self.subTest(depth=depth):
                index = self.index_manager.create_index(parent, row)
                result = self.index_manager.to_sequence(index)
                self.assertEqual(result, sequence[:depth+1])
                parent = index

    def test_complex_index_sequence_round_trip(self):
        parent = Root
        for depth, row in enumerate([5, 6, 7, 8, 9, 10]):
            with self.subTest(depth=depth):
                index = self.index_manager.create_index(parent, row)
                sequence = self.index_manager.to_sequence(index)
                result = self.index_manager.from_sequence(sequence)
                self.assertIs(result, index)
                parent = index

    def test_complex_index_id_round_trip(self):
        sequence = (5, 6, 7, 8, 9, 10)
        parent = Root
        for depth, row in enumerate(sequence):
            with self.subTest(depth=depth):
                index = self.index_manager.create_index(parent, row)
                id = self.index_manager.id(index)
                self.assertIsInstance(id, int)
                result = self.index_manager.from_id(id)
                self.assertIs(result, index)
                parent = index



# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


""" View of an ActionManager drawn as a rectangle of buttons.
"""

import wx
from pyface.widget import Widget

from traits.api import Bool, Dict, Int, List, Tuple

# HTML templates.
# FIXME : Not quite the right color.
HTML = """

<html>
  <body bgcolor='#cccccc'>
    %s
  </body>
</html>

"""

PART = """<wxp module="wx" class="Panel"><param name="id" value="%s"><param name="size" value="%s"></wxp>"""


class ToolPalette(Widget):

    tools = List()

    id_tool_map = Dict()

    tool_id_to_button_map = Dict()

    button_size = Tuple((25, 25), Int, Int)

    is_realized = Bool(False)

    tool_listeners = Dict()

    # Maps a button id to its tool id.
    button_tool_map = Dict()

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, parent, **traits):
        """ Creates a new tool palette. """

        # Base class constructor.
        super(ToolPalette, self).__init__(**traits)

        # Create the toolkit-specific control that represents the widget.
        self.control = self._create_control(parent)

        return

    # ------------------------------------------------------------------------
    # ToolPalette interface.
    # ------------------------------------------------------------------------

    def add_tool(self, label, bmp, kind, tooltip, longtip):
        """ Add a tool with the specified properties to the palette.

        Return an id that can be used to reference this tool in the future.
        """

        wxid = wx.NewIdRef()
        params = (wxid, label, bmp, kind, tooltip, longtip)
        self.tools.append(params)
        self.id_tool_map[wxid] = params

        if self.is_realized:
            self._reflow()

        return wxid

    def toggle_tool(self, id, checked):
        """ Toggle the tool identified by 'id' to the 'checked' state.

        If the button is a toggle or radio button, the button will be checked
        if the 'checked' parameter is True; unchecked otherwise.  If the button
        is a standard button, this method is a NOP.
        """

        button = self.tool_id_to_button_map.get(id, None)
        if button is not None and hasattr(button, "SetToggle"):
            button.SetToggle(checked)

    def enable_tool(self, id, enabled):
        """ Enable or disable the tool identified by 'id'. """

        button = self.tool_id_to_button_map.get(id, None)
        if button is not None:
            button.SetEnabled(enabled)

    def on_tool_event(self, id, callback):
        """ Register a callback for events on the tool identified by 'id'. """

        callbacks = self.tool_listeners.setdefault(id, [])
        callbacks.append(callback)

    def realize(self):
        """ Realize the control so that it can be displayed. """

        self.is_realized = True
        self._reflow()

    def get_tool_state(self, id):
        """ Get the toggle state of the tool identified by 'id'. """

        button = self.tool_id_to_button_map.get(id, None)
        if hasattr(button, "GetToggle"):
            if button.GetToggle():
                state = 1
            else:
                state = 0
        else:
            state = 0

        return state

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):

        html_window = wx.html.HtmlWindow(parent, -1, style=wx.CLIP_CHILDREN)

        return html_window

    def _reflow(self):
        """ Reflow the layout. """

        # Create a bit of html for each tool.
        parts = []
        for param in self.tools:
            parts.append(PART % (str(param[0]), self.button_size))

        # Create the entire html page.
        html = HTML % "".join(parts)

        # Set the HTML on the widget.  This will create all of the buttons.
        self.control.SetPage(html)

        for param in self.tools:
            self._initialize_tool(param)

    def _initialize_tool(self, param):
        """ Initialize the tool palette button. """

        wxid, label, bmp, kind, tooltip, longtip = param

        panel = self.control.FindWindowById(wxid)

        sizer = wx.BoxSizer(wx.VERTICAL)
        panel.SetSizer(sizer)
        panel.SetAutoLayout(True)
        panel.SetWindowStyleFlag(wx.CLIP_CHILDREN)

        from wx.lib.buttons import GenBitmapToggleButton, GenBitmapButton

        if kind == "radio":
            button = GenBitmapToggleButton(
                panel, -1, None, size=self.button_size
            )

        else:
            button = GenBitmapButton(panel, -1, None, size=self.button_size)

        self.button_tool_map[button.GetId()] = wxid
        self.tool_id_to_button_map[wxid] = button
        panel.Bind(wx.EVT_BUTTON, self._on_button, button)
        button.SetBitmapLabel(bmp)
        button.SetToolTip(label)
        sizer.Add(button, 0, wx.EXPAND)

    def _on_button(self, event):

        button_id = event.GetId()
        tool_id = self.button_tool_map.get(button_id, None)
        if tool_id is not None:
            for listener in self.tool_listeners.get(tool_id, []):
                listener(event)

        return
